@@ -2,17 +2,19 @@
 from abc import ABCMeta
 from collections.abc import Sequence
 from enum import Enum
-from typing import Iterable, Mapping, Optional, Union
+from typing import Iterable, Mapping, MutableMapping, Optional, Union
 from xml.etree import ElementTree as et  # nosec - We are creating, not parsing XML.
 
 from furl import furl
-from mbtest.imposters.base import JsonSerializable, JsonStructure
+
+from mbtest.imposters.base import Injecting, JsonSerializable, JsonStructure
 from mbtest.imposters.behaviors import Copy, Lookup
+from mbtest.imposters.predicates import Predicate
 
 
 class BaseResponse(JsonSerializable, metaclass=ABCMeta):
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "BaseResponse":
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "BaseResponse":
         if "is" in structure and "_behaviors" in structure:
             return Response.from_structure(structure)
         elif "is" in structure and "data" in structure["is"]:
@@ -87,38 +89,38 @@ class Response(BaseResponse):
 
     def _is_structure(self) -> JsonStructure:
         is_structure = {"statusCode": self.status_code, "_mode": self.mode.value}
-        self._add_if_true(is_structure, "body", self.body)
-        self._add_if_true(is_structure, "headers", self.headers)
+        self.add_if_true(is_structure, "body", self.body)
+        self.add_if_true(is_structure, "headers", self.headers)
         return is_structure
 
     def _behaviors_structure(self) -> JsonStructure:
-        behaviors = {}  # type: JsonStructure
-        self._add_if_true(behaviors, "wait", self.wait)
-        self._add_if_true(behaviors, "repeat", self.repeat)
-        self._add_if_true(behaviors, "decorate", self.decorate)
-        self._add_if_true(behaviors, "shellTransform", self.shell_transform)
+        behaviors: JsonStructure = {}
+        self.add_if_true(behaviors, "wait", self.wait)
+        self.add_if_true(behaviors, "repeat", self.repeat)
+        self.add_if_true(behaviors, "decorate", self.decorate)
+        self.add_if_true(behaviors, "shellTransform", self.shell_transform)
         if self.copy:
             behaviors["copy"] = [c.as_structure() for c in self.copy]
         if self.lookup:
             behaviors["lookup"] = [lookup.as_structure() for lookup in self.lookup]
         return behaviors
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "Response":
-        response = Response()
-        response._fields_from_structure(structure)
-        behaviors = structure.get("_behaviors")
-        response._set_if_in_dict(behaviors, "wait", "wait")
-        response._set_if_in_dict(behaviors, "repeat", "repeat")
-        response._set_if_in_dict(behaviors, "decorate", "decorate")
-        response._set_if_in_dict(behaviors, "shellTransform", "shell_transform")
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "Response":
+        response = cls()
+        response.fields_from_structure(structure)
+        behaviors = structure.get("_behaviors", {})
+        response.set_if_in_dict(behaviors, "wait", "wait")
+        response.set_if_in_dict(behaviors, "repeat", "repeat")
+        response.set_if_in_dict(behaviors, "decorate", "decorate")
+        response.set_if_in_dict(behaviors, "shellTransform", "shell_transform")
         if "copy" in behaviors:
             response.copy = [Copy.from_structure(c) for c in behaviors["copy"]]
         if "lookup" in behaviors:
             response.lookup = [Lookup.from_structure(lookup) for lookup in behaviors["lookup"]]
         return response
 
-    def _fields_from_structure(self, structure: JsonStructure) -> None:
+    def fields_from_structure(self, structure: JsonStructure) -> None:
         inner = structure["is"]
         if "body" in inner:
             self._body = inner["body"]
@@ -136,9 +138,9 @@ class TcpResponse(BaseResponse):
     def as_structure(self) -> JsonStructure:
         return {"is": {"data": self.data}}
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "TcpResponse":
-        return TcpResponse(data=structure["is"]["data"])
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "TcpResponse":
+        return cls(data=structure["is"]["data"])
 
 
 class Proxy(BaseResponse):
@@ -159,33 +161,44 @@ class Proxy(BaseResponse):
         to: Union[furl, str],
         wait: Optional[int] = None,
         inject_headers: Optional[Mapping[str, str]] = None,
-        mode: "Proxy.Mode" = Mode.ALWAYS,
+        mode: "Proxy.Mode" = Mode.ONCE,
+        predicate_generators: Optional[Iterable["PredicateGenerator"]] = None,
     ) -> None:
         self.to = to
         self.wait = wait
         self.inject_headers = inject_headers
         self.mode = mode
+        self.predicate_generators = predicate_generators if predicate_generators is not None else []
 
     def as_structure(self) -> JsonStructure:
         proxy = {
             "to": self.to.url if isinstance(self.to, furl) else self.to,
             "mode": self.mode.value,
         }
-        self._add_if_true(proxy, "injectHeaders", self.inject_headers)
+        self.add_if_true(proxy, "injectHeaders", self.inject_headers)
+        self.add_if_true(
+            proxy, "predicateGenerators", [pg.as_structure() for pg in self.predicate_generators]
+        )
         response = {"proxy": proxy}
         if self.wait:
             response["_behaviors"] = {"wait": self.wait}
         return response
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "Proxy":
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "Proxy":
         proxy_structure = structure["proxy"]
-        proxy = Proxy(
+        proxy = cls(
             to=furl(proxy_structure["to"]),
             inject_headers=proxy_structure["injectHeaders"]
             if "injectHeaders" in proxy_structure
             else None,
             mode=Proxy.Mode(proxy_structure["mode"]),
+            predicate_generators=[
+                PredicateGenerator.from_structure(pg)
+                for pg in proxy_structure["predicateGenerators"]
+            ]
+            if "predicateGenerators" in proxy_structure
+            else None,
         )
         wait = structure.get("_behaviors", {}).get("wait")
         if wait:
@@ -193,7 +206,40 @@ class Proxy(BaseResponse):
         return proxy
 
 
-class InjectionResponse(BaseResponse):
+class PredicateGenerator(JsonSerializable):
+    def __init__(
+        self,
+        path: bool = False,
+        query: Union[bool, Mapping[str, str]] = False,
+        operator: Predicate.Operator = Predicate.Operator.EQUALS,
+        case_sensitive: bool = True,
+    ):
+        self.path = path
+        self.query = query
+        self.operator = operator
+        self.case_sensitive = case_sensitive
+
+    def as_structure(self) -> JsonStructure:
+        matches: MutableMapping[str, str] = {}
+        self.add_if_true(matches, "path", self.path)
+        self.add_if_true(matches, "query", self.query)
+        predicate = {"caseSensitive": self.case_sensitive, "matches": matches}
+        return predicate
+
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "PredicateGenerator":
+        path = structure["matches"].get("path", None)
+        query = structure["matches"].get("query", None)
+        operator = (
+            Predicate.Operator[structure["operator"]]
+            if "operator" in structure
+            else Predicate.Operator.EQUALS
+        )
+        case_sensitive = structure.get("caseSensitive", False)
+        return cls(path=path, query=query, operator=operator, case_sensitive=case_sensitive)
+
+
+class InjectionResponse(BaseResponse, Injecting):
     """Represents a `Mountebank injection response <http://www.mbtest.org/docs/api/injection>`_.
 
     Injection requires Mountebank version 2.0 or higher.
@@ -201,12 +247,6 @@ class InjectionResponse(BaseResponse):
     :param inject: JavaScript function to inject .
     """
 
-    def __init__(self, inject: str) -> None:
-        self.inject = inject
-
-    def as_structure(self) -> JsonStructure:
-        return {"inject": self.inject}
-
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "InjectionResponse":
-        return InjectionResponse(inject=structure["inject"])
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "InjectionResponse":
+        return cls(inject=structure["inject"])
