@@ -35,15 +35,54 @@ class Stub(JsonSerializable):
             "responses": [response.as_structure() for response in self.responses],
         }
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "Stub":
-        responses = []  # type: List[Union[Proxy, Response]]
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "Stub":
+        responses: List[Union[Proxy, Response]] = []
         for response in structure.get("responses", ()):
             if "proxy" in response:
                 responses.append(Proxy.from_structure(response))
             else:
                 responses.append(Response.from_structure(response))
-        return Stub(
-            [Predicate.from_structure(predicate) for predicate in structure.get("predicates", ())],
+        return cls(
+            [
+                BasePredicate.from_structure(predicate)
+                for predicate in structure.get("predicates", ())
+            ],
             responses,
         )
+
+
+class AddStub(JsonSerializable):
+    """Represents a `Mountebank add stub request <http://www.mbtest.org/docs/api/overview#add-stub>`.
+    To add new stab to an existing imposter.
+
+    :param index: The index in imposter stubs array.
+     If you leave off the index field, the stub will be added to the end of the existing stubs array.
+    :param stub: The stub that will be added to the existing stubs array
+    """
+
+    def __init__(
+        self,
+        stub: Stub = None,
+        index: int = None,
+    ) -> None:
+        self.index = index
+        if stub:
+            self.stub = stub
+        else:
+            self.stub = Stub()
+
+    def as_structure(self) -> JsonStructure:
+        structure = {
+            "stub": self.stub.as_structure(),
+        }
+        if self.index is not None:
+            structure["index"] = self.index
+        return structure
+
+    @staticmethod
+    def from_structure(structure: JsonStructure) -> "AddStub":
+        return AddStub(
+            index=structure.get("index"),
+            stub=Stub().from_structure(structure.get("stub")),
+        )
